/*

 DC Receiver special ADC driver

 OH2NLT 13.01.2004
 
 A/D converter must be kept closed when not operating.
 This approach frees few digital I/O bits for use.

 Code size optimization 18.01.2005
 16F88 conversion added 28.08.2005
 Overflow protection 09.05.2006

*/

//*****************************************
// Convert one channel returns integer value
// chnum selects channel
//*****************************************
// Start conversion & return with result
void convert_adc(void)
	{
	delayu(250);
	GO_DONE = 1;						// Set flag
	asm("nop");

	while (GO_DONE);					// Wait for conversion complete flag to be reset
	}

// Set up A/D converter, measure CH0 & CH1
// Free digital I/O's after operation
void measure_all(void)
	{
	long v;

#ifdef F88
	ANSEL = ANSEL_MEAS;					// Select configure F88 I/O
#else
	ADCON1 = ADCON1_MEAS;				// Select mode & configure I/O
#endif

	ADCON0 = ADCON0_CH0;				// Select Channel 0 & turn A/D system on
	convert_adc();
	bat.batregs[2] = ADRESL;
	bat.batregs[3] = ADRESH;

	ADCON0 = ADCON0_CH1;				// Select Channel 1 & turn A/D system on
	convert_adc();
	smeter = ADRESL;					// Save S-meter data. Only LSB is used.

	if(ADRESH != 0)
		smeter = 255;					// If overflow, saturate to 255

	ADCON0 = ADCON0_OFF;				// Shut down A/D system

#ifdef F88
	ANSEL = ANSEL_INIT;					// PORTA pins digital I/O
#else
	ADCON1 = ADCON1_INIT;				// PORTA pins digital I/O
#endif

	if(ad_inhibit)						// Inhibit voltmeter sampling if tone was active
		{
		ad_inhibit--;
		return;
		}

	if(samples)
		{
		batt_avg += BATT_VOLTAGE;
		samples--;
		}
	else
		{
		v = (long)(batt_avg * MULTIPLIER) / (long)(BATT_FACTOR);
		batt_voltage = (int)(v);
		batt_avg = 0;
		samples = AVG_SAMPLES;
		}
	}

