/*

 DC Receiver controller Timing & Interrupt routines

 OH2NLT / Juha Niinikoski 12.01.2005

 Code size optimization 18.01.2004
 New Rotary Encoder logic 20.01.2005
 Encoder debounce filter 03.01.2005

*/

// Delays calculated for 20 MHz clock

static char oldpw;						// Encoder old state
static char pw;							// Encoder current state
static char pw_change;					// State machine variable
static unsigned char old_portb_copy;	// Old encoder bits for debounce
static char _mode;
unsigned int button_count;				// Button Pressed counter
unsigned char operating_mode;			// Transceiver operating mode, cleared in main()
unsigned char exit_mode;				// Operating mode exit code, reset in main()

// Init timers & interrupt system
void Init_timers(void)
	{
//	TMR1ON = 0;		// Stop TMR1, TMR1 is reserved for delays and waits
//	TMR1CS = 0;		// Internal clock
//	T1CKPS0 = 0;	// Prescaler = 4 ==> TMR1 Clock = 1,25 MHz (800ns tick)
//	T1CKPS1 = 1;
	T1CON = 0x20;	// Same as above, but fewer instructions

	PR2 = 0xF9;		// Generate 1 ms IRQ. 20 MHz / 4 = 5 MHz / 4 / 250 / 5 = 1 mS
	T2CON = 0x21;	// Prescaler = 4 = 1250kHz clock for TMR2, postscaler = 5
	TMR2ON = 1;		// Start TIMER2
	TMR2IE = 1;		// Enable TMR2 IRQ
	oldpw = 3;
	PEIE = 1;		// Enable peripheral IRQs
	GIE = 1;		// Enable global IRQs
	}

// IRQ handlers, TMR2 is only source
// TMR2 general timing IRQ
static void interrupt tmr2_int(void)
	{
	TMR2IF = 0;						// Reset IRQ flag

// Pulse wheel logic
	TRISB = TRISB_READ;				// Change PORTB to inputs, check main loop routines !

	if(ms_timer)
		ms_timer--;					// Do 1ms wait timer

	asm("nop");						// Few us capacitor (bus) load time
	asm("nop");
	asm("nop");						// Let signal stabilize
	portb_copy = PORTB;				// Read & save port bits

	if(portb_copy == old_portb_copy) // Debounce filter
		pw = (portb_copy >> 6) & 3;	 // Shift RB7, RB6 down & mask

	old_portb_copy = portb_copy;
	TRISB = TRISB_INIT;				// Reset PORTB directions to its initial state

	if(pw != oldpw)					// Second generation logic
		{
		pw_change = pw ^ oldpw;		// Filter changed bit

// If you have High / Low dent encoder leave both sections active
		if(oldpw == 3 || oldpw == 0)
//		if(oldpw == 3)				// High at dents type encoder
			{
			if(pw_change == 1)		// Changing the pwctr to literal assignments of +1 or -1 takes 3 more bytes.
				pwctr++;

			if(pw_change == 2)
				pwctr--;
			}

		oldpw = pw;
		}

	if(BUTTON == 0)					// Button pressed,
		{
		button_count++;				// so increment the button count.

		if(button_count > LONG_PRESS)	// This prevents someone from holding the button down for 65,536mS
			button_count = LONG_PRESS;	// and then finding that they might not get the right exit code.
		}
	else							// Button released.
		{
		if(button_count >= SHORT_PRESS)
			_mode++;				// Set button state to 1

		if(button_count >= MEDIUM_PRESS)
			_mode++;				// Set button state to 2

		if(button_count >= LONG_PRESS)
			_mode++;				// Set button state to 3

		button_count = 0;			// Otherwise, it is just a glitch, so reset the counter
		}

	if(operating_mode == 0)			// Starting state, no pending button actions.
		operating_mode = _mode;
	else if(exit_mode == 0)			// Fixes exit mode bug.
			exit_mode = _mode;

	_mode = 0;

	if(blink_timer)
		blink_timer--;
	else
		blink_timer = BLINK_PERIOD;

//	dsb_mode = ((portb_copy & TX) == TX);
	}

// Microsecond delay, small delays not accurate
// Overhead added to requested delay about 12uS
void delayu(unsigned int ticks)
	{
	union
		{
		int t1;
		unsigned char t1regs[2];
		} timex;

	timex.t1 = 0xFFFF - ticks;
	TMR1ON = 0;					// Stop timer & set new value
	TMR1IF = 0;
	TMR1H = timex.t1regs[1];	// High part must go first
	TMR1L = timex.t1regs[0];
	TMR1ON = 1;

	while(!TMR1IF);				// Wait for timer to expire...
	}

// Millisecond delays
void delaym(unsigned int millisec)
	{
	while(millisec)				// Auto-decrement here uses 1 more byte.
		{
		delayu(1000);
		millisec --;
		}
	}	

